import { resolve } from "@std/path/posix";
import type View from "./views/View.ts";
import config from "./config.ts";

export type Plugin = (this: View, ...args: unknown[]) => unknown;

const plugins = new Map<string, Plugin>();
let loaded = false;

export function getPlugin(name: string) {
    const res = plugins.get(name);
    if (res == null) throw new Error(`Plugin ${name} not loaded (maybe you forgot to include it in the config?)`);
    else return res;
}

export async function loadPlugins() {
    if (loaded) return;
    loaded = true;

    for (const plugin of config.plugins) {
        const func = (await import(resolve(config.libs + "/" + plugin + ".ts"))).default;
        plugins.set(plugin, func);
    }
}