import frontMatter from "markdown-it-front-matter";
import markdown from "markdown-it";
import hljs from "highlight.js";
import View, { type Locals, type Metadata } from "./View.ts";
import { resolveView } from "./resolution.ts";
import { parse as parseYaml } from "@std/yaml";

export default class MarkdownView extends View {
    public static metadata: Metadata;
    public static readonly renderer = markdown({
        html: true,
        xhtmlOut: false,
        breaks: false,
        langPrefix: 'language-',
        linkify: true,
        typographer: false,
        quotes: '“”‘’',
        highlight: (str: string, lang: string) => {
            if (lang && hljs.getLanguage(lang)) {
                try {
                    return hljs.highlight(str, { language: lang }).value;
                }
                catch {}
            }

            return "";
        }
    }).use(frontMatter, (v: Metadata) => this.metadata = v);

    public override onMetadata() {
        MarkdownView.renderMarkdown(this.src, { metadata: true }, false);
        return MarkdownView.metadata;
    }

    public override onRender(locals: Locals, full = false) {
        locals = this.locals(locals);

        const text = MarkdownView.renderMarkdown(this.src, locals, full);

        if (full && locals.page) {
            const parent = resolveView(String(locals.page), false, true);
            return parent?.render({ ...locals, text }, full) ?? text;
        }
        else return text;
    }

    public static renderMarkdown(text: string, _locals: object, _full: boolean): string {
        this.metadata = "{}" as any;
        const res = this.renderer.render(text);
        this.metadata = parseYaml(this.metadata as any) as Metadata;
        return res;
        // return (await MarkdownView.renderer.render(text, { metadata: true })).value;
    }
}
