import View, { type Locals } from "./View.ts";
import config from "../config.ts";
import { compile } from "pug";
import MarkdownView from "./MarkdownView.ts";
import { invoke, renderView } from "./resolution.ts";

export default class PugView extends View {
    public override onMetadata() {
        const src = this.src;
        const lines: string[] = [];

        for (const match of src.matchAll(/\/\/---\n((?:    .+?\n)*)\/\/---/g)) {
            lines.push(...match[1].split("\n").map(v => v.slice(4)));
        }

        return lines.join("\n") ?? "";
    }

    public override onRender(locals: Locals, full: boolean): string {
        locals = this.locals(locals);
        const src = this.src;

        const func = compile(src, {
            basedir: config.views,
            compileDebug: false,
            debug: config.debug,
            pretty: config.debug,
            doctype: 'HTML',
            filters: {
                md: (src: string) => MarkdownView.renderMarkdown(src, {}, false),
            },
            globals: [],
        });

        const res = func({
            ...locals,
            invoke: invoke.bind(View, this),
        });

        if (full && !src.match(/^extends\s/g) && !src.match(/^doctype\s/g)) {
            if (locals.page != null) return renderView(String(locals.page), { ...locals, text: res }, false, true, true)?.text ?? res;
        }

        return res;
    }
}
